@@ -1,27 +1,35 @@
 use super::{
     as_handle::AsHandle,
     bind::{CDataMut, DelayedInput, HasDataType},
-    buffer::{buf_ptr, clamp_small_int, mut_buf_ptr},
+    buffer::{clamp_small_int, mut_buf_ptr},
     column_description::{ColumnDescription, Nullability},
     data_type::DataType,
     drop_handle,
+    sql_char::{binary_length, is_truncated_bin, resize_to_fit_without_tz},
     sql_result::ExtSqlReturn,
-    CData, SqlResult,
+    CData, SqlChar, SqlResult, SqlText,
 };
 use odbc_sys::{
     Desc, FreeStmtOption, HDbc, HStmt, Handle, HandleType, Len, ParamType, Pointer, SQLBindCol,
-    SQLBindParameter, SQLCloseCursor, SQLColAttributeW, SQLColumnsW, SQLDescribeColW,
-    SQLDescribeParam, SQLExecDirectW, SQLExecute, SQLFetch, SQLFreeStmt, SQLGetData,
-    SQLNumResultCols, SQLParamData, SQLPrepareW, SQLPutData, SQLSetStmtAttrW, SQLTablesW,
-    SqlDataType, SqlReturn, StatementAttribute, ULen,
+    SQLBindParameter, SQLCloseCursor, SQLDescribeParam, SQLExecute, SQLFetch, SQLFreeStmt,
+    SQLGetData, SQLNumResultCols, SQLParamData, SQLPutData, SqlDataType, SqlReturn,
+    StatementAttribute, IS_POINTER,
 };
-use std::{
-    ffi::c_void,
-    marker::PhantomData,
-    mem::ManuallyDrop,
-    ptr::{null, null_mut},
+use std::{ffi::c_void, marker::PhantomData, mem::ManuallyDrop, ptr::null_mut};
+
+#[cfg(feature = "narrow")]
+use odbc_sys::{
+    SQLColAttribute as sql_col_attribute, SQLColumns as sql_columns,
+    SQLDescribeCol as sql_describe_col, SQLExecDirect as sql_exec_direc, SQLPrepare as sql_prepare,
+    SQLSetStmtAttr as sql_set_stmt_attr, SQLTables as sql_tables,
+};
+
+#[cfg(not(feature = "narrow"))]
+use odbc_sys::{
+    SQLColAttributeW as sql_col_attribute, SQLColumnsW as sql_columns,
+    SQLDescribeColW as sql_describe_col, SQLExecDirectW as sql_exec_direc,
+    SQLPrepareW as sql_prepare, SQLSetStmtAttrW as sql_set_stmt_attr, SQLTablesW as sql_tables,
 };
-use widestring::U16Str;
 
 /// Wraps a valid (i.e. successfully allocated) ODBC statement handle.
 pub struct StatementImpl<'s> {
@@ -149,12 +157,17 @@ pub trait Statement: AsHandle {
     /// # Safety
     ///
     /// `num_rows` must not be moved and remain valid, as long as it remains bound to the cursor.
-    unsafe fn set_num_rows_fetched(&mut self, num_rows: Option<&mut ULen>) -> SqlResult<()> {
+    unsafe fn set_num_rows_fetched(&mut self, num_rows: Option<&mut usize>) -> SqlResult<()> {
         let value = num_rows
-            .map(|r| r as *mut ULen as Pointer)
+            .map(|r| r as *mut usize as Pointer)
             .unwrap_or_else(null_mut);
-        SQLSetStmtAttrW(self.as_sys(), StatementAttribute::RowsFetchedPtr, value, 0)
-            .into_sql_result("SQLSetStmtAttrW")
+        sql_set_stmt_attr(
+            self.as_sys(),
+            StatementAttribute::RowsFetchedPtr,
+            value,
+            IS_POINTER,
+        )
+        .into_sql_result("SQLSetStmtAttr")
     }
 
     /// Fetch a column description using the column index.
@@ -181,7 +194,7 @@ pub trait Statement: AsHandle {
         let mut nullable = odbc_sys::Nullability::UNKNOWN;
 
         let res = unsafe {
-            SQLDescribeColW(
+            sql_describe_col(
                 self.as_sys(),
                 column_number,
                 mut_buf_ptr(name),
@@ -192,7 +205,7 @@ pub trait Statement: AsHandle {
                 &mut decimal_digits,
                 &mut nullable,
             )
-            .into_sql_result("SQLDescribeColW")
+            .into_sql_result("SQLDescribeCol")
         };
 
         if res.is_err() {
@@ -226,17 +239,17 @@ pub trait Statement: AsHandle {
     /// # Return
     ///
     /// Returns `true` if execution requires additional data from delayed parameters.
-    unsafe fn exec_direct(&mut self, statement_text: &U16Str) -> SqlResult<bool> {
-        match SQLExecDirectW(
+    unsafe fn exec_direct(&mut self, statement: &SqlText) -> SqlResult<bool> {
+        match sql_exec_direc(
             self.as_sys(),
-            buf_ptr(statement_text.as_slice()),
-            statement_text.len().try_into().unwrap(),
+            statement.ptr(),
+            statement.len_char().try_into().unwrap(),
         ) {
             SqlReturn::NEED_DATA => SqlResult::Success(true),
             // A searched update or delete statement that does not affect any rows at the data
             // source.
             SqlReturn::NO_DATA => SqlResult::Success(false),
-            other => other.into_sql_result("SQLExecDirectW").on_success(|| false),
+            other => other.into_sql_result("SQLExecDirect").on_success(|| false),
         }
     }
 
@@ -248,15 +261,15 @@ pub trait Statement: AsHandle {
     /// Send an SQL statement to the data source for preparation. The application can include one or
     /// more parameter markers in the SQL statement. To include a parameter marker, the application
     /// embeds a question mark (?) into the SQL string at the appropriate position.
-    fn prepare(&mut self, statement_text: &U16Str) -> SqlResult<()> {
+    fn prepare(&mut self, statement: &SqlText) -> SqlResult<()> {
         unsafe {
-            SQLPrepareW(
+            sql_prepare(
                 self.as_sys(),
-                buf_ptr(statement_text.as_slice()),
-                statement_text.len().try_into().unwrap(),
+                statement.ptr(),
+                statement.len_char().try_into().unwrap(),
             )
         }
-        .into_sql_result("SQLPrepareW")
+        .into_sql_result("SQLPrepare")
     }
 
     /// Executes a statement prepared by `prepare`. After the application processes or discards the
@@ -301,13 +314,13 @@ pub trait Statement: AsHandle {
     /// specified amount of rows.
     unsafe fn set_row_array_size(&mut self, size: usize) -> SqlResult<()> {
         assert!(size > 0);
-        SQLSetStmtAttrW(
+        sql_set_stmt_attr(
             self.as_sys(),
             StatementAttribute::RowArraySize,
             size as Pointer,
             0,
         )
-        .into_sql_result("SQLSetStmtAttrW")
+        .into_sql_result("SQLSetStmtAttr")
     }
 
     /// Specifies the number of values for each parameter. If it is greater than 1, the data and
@@ -320,13 +333,13 @@ pub trait Statement: AsHandle {
     /// statement is executed.
     unsafe fn set_paramset_size(&mut self, size: usize) -> SqlResult<()> {
         assert!(size > 0);
-        SQLSetStmtAttrW(
+        sql_set_stmt_attr(
             self.as_sys(),
             StatementAttribute::ParamsetSize,
             size as Pointer,
             0,
         )
-        .into_sql_result("SQLSetStmtAttrW")
+        .into_sql_result("SQLSetStmtAttr")
     }
 
     /// Sets the binding type to columnar binding for batch cursors.
@@ -339,24 +352,24 @@ pub trait Statement: AsHandle {
     /// It is the callers responsibility to ensure that the bound buffers match the memory layout
     /// specified by this function.
     unsafe fn set_row_bind_type(&mut self, row_size: usize) -> SqlResult<()> {
-        SQLSetStmtAttrW(
+        sql_set_stmt_attr(
             self.as_sys(),
             StatementAttribute::RowBindType,
             row_size as Pointer,
             0,
         )
-        .into_sql_result("SQLSetStmtAttrW")
+        .into_sql_result("SQLSetStmtAttr")
     }
 
     fn set_metadata_id(&mut self, metadata_id: bool) -> SqlResult<()> {
         unsafe {
-            SQLSetStmtAttrW(
+            sql_set_stmt_attr(
                 self.as_sys(),
                 StatementAttribute::MetadataId,
-                metadata_id as u8 as Pointer,
+                metadata_id as usize as Pointer,
                 0,
             )
-            .into_sql_result("SQLSetStmtAttrW")
+            .into_sql_result("SQLSetStmtAttr")
         }
     }
 
@@ -518,47 +531,48 @@ pub trait Statement: AsHandle {
 
     /// The column alias, if it applies. If the column alias does not apply, the column name is
     /// returned. If there is no column name or a column alias, an empty string is returned.
-    fn col_name(&self, column_number: u16, buf: &mut Vec<u16>) -> SqlResult<()> {
+    fn col_name(&self, column_number: u16, buffer: &mut Vec<SqlChar>) -> SqlResult<()> {
         // String length in bytes, not characters. Terminating zero is excluded.
         let mut string_length_in_bytes: i16 = 0;
         // Let's utilize all of `buf`s capacity.
-        buf.resize(buf.capacity(), 0);
+        buffer.resize(buffer.capacity(), 0);
         unsafe {
-            let mut res = SQLColAttributeW(
+            let mut res = sql_col_attribute(
                 self.as_sys(),
                 column_number,
                 Desc::Name,
-                mut_buf_ptr(buf) as Pointer,
-                (buf.len() * 2).try_into().unwrap(),
+                mut_buf_ptr(buffer) as Pointer,
+                binary_length(buffer).try_into().unwrap(),
                 &mut string_length_in_bytes as *mut i16,
                 null_mut(),
             )
-            .into_sql_result("SQLColAttributeW");
+            .into_sql_result("SQLColAttribute");
 
             if res.is_err() {
                 return res;
             }
 
-            if clamp_small_int(buf.len() * 2) < string_length_in_bytes + 2 {
+            if is_truncated_bin(buffer, string_length_in_bytes.try_into().unwrap()) {
                 // If we could rely on every ODBC driver sticking to the specifcation it would
-                // probably best to resize by `string_length_in_bytes / 2 + 1`. Yet i.e. SQLite
+                // probably best to resize by `string_length_in_bytes / 2 + 1`. Yet e.g. SQLite
                 // seems to report the length in characters, so to work with a wide range of DB
                 // systems, and since buffers for names are not expected to become super large we
                 // ommit the division by two here.
-                buf.resize((string_length_in_bytes + 1).try_into().unwrap(), 0);
-                res = SQLColAttributeW(
+                buffer.resize((string_length_in_bytes + 1).try_into().unwrap(), 0);
+
+                res = sql_col_attribute(
                     self.as_sys(),
                     column_number,
                     Desc::Name,
-                    mut_buf_ptr(buf) as Pointer,
-                    (buf.len() * 2).try_into().unwrap(),
+                    mut_buf_ptr(buffer) as Pointer,
+                    binary_length(buffer).try_into().unwrap(),
                     &mut string_length_in_bytes as *mut i16,
                     null_mut(),
                 )
-                .into_sql_result("SQLColAttributeW");
+                .into_sql_result("SQLColAttribute");
             }
             // Resize buffer to exact string length without terminal zero
-            buf.resize(((string_length_in_bytes + 1) / 2).try_into().unwrap(), 0);
+            resize_to_fit_without_tz(buffer, string_length_in_bytes.try_into().unwrap());
 
             res
         }
@@ -569,7 +583,7 @@ pub trait Statement: AsHandle {
     /// It is the callers responsibility to ensure that `attribute` refers to a numeric attribute.
     unsafe fn numeric_col_attribute(&self, attribute: Desc, column_number: u16) -> SqlResult<Len> {
         let mut out: Len = 0;
-        SQLColAttributeW(
+        sql_col_attribute(
             self.as_sys(),
             column_number,
             attribute,
@@ -578,7 +592,7 @@ pub trait Statement: AsHandle {
             null_mut(),
             &mut out as *mut Len,
         )
-        .into_sql_result("SQLColAttributeW")
+        .into_sql_result("SQLColAttribute")
         .on_success(|| out)
     }
 
@@ -637,24 +651,24 @@ pub trait Statement: AsHandle {
     /// Executes a columns query using this statement handle.
     fn columns(
         &mut self,
-        catalog_name: &U16Str,
-        schema_name: &U16Str,
-        table_name: &U16Str,
-        column_name: &U16Str,
+        catalog_name: &SqlText,
+        schema_name: &SqlText,
+        table_name: &SqlText,
+        column_name: &SqlText,
     ) -> SqlResult<()> {
         unsafe {
-            SQLColumnsW(
+            sql_columns(
                 self.as_sys(),
-                buf_ptr(catalog_name.as_slice()),
-                catalog_name.len().try_into().unwrap(),
-                buf_ptr(schema_name.as_slice()),
-                schema_name.len().try_into().unwrap(),
-                buf_ptr(table_name.as_slice()),
-                table_name.len().try_into().unwrap(),
-                buf_ptr(column_name.as_slice()),
-                column_name.len().try_into().unwrap(),
+                catalog_name.ptr(),
+                catalog_name.len_char().try_into().unwrap(),
+                schema_name.ptr(),
+                schema_name.len_char().try_into().unwrap(),
+                table_name.ptr(),
+                table_name.len_char().try_into().unwrap(),
+                column_name.ptr(),
+                column_name.len_char().try_into().unwrap(),
             )
-            .into_sql_result("SQLColumnsW")
+            .into_sql_result("SQLColumns")
         }
     }
 
@@ -666,38 +680,24 @@ pub trait Statement: AsHandle {
     /// otherwise a NulPointer error is emitted.
     fn tables(
         &mut self,
-        catalog_name: Option<&U16Str>,
-        schema_name: Option<&U16Str>,
-        table_name: Option<&U16Str>,
-        table_type: Option<&U16Str>,
+        catalog_name: &SqlText,
+        schema_name: &SqlText,
+        table_name: &SqlText,
+        table_type: &SqlText,
     ) -> SqlResult<()> {
-        // Convert each filter into a pair of buffer pointer and buffer length.
-        let to_buf = |filter: Option<&U16Str>| {
-            if let Some(text) = filter {
-                (buf_ptr(text.as_slice()), text.len().try_into().unwrap())
-            } else {
-                (null(), 0i16)
-            }
-        };
-
-        let catalog = to_buf(catalog_name);
-        let schema = to_buf(schema_name);
-        let table = to_buf(table_name);
-        let type_ = to_buf(table_type);
-
         unsafe {
-            SQLTablesW(
+            sql_tables(
                 self.as_sys(),
-                catalog.0,
-                catalog.1,
-                schema.0,
-                schema.1,
-                table.0,
-                table.1,
-                type_.0,
-                type_.1,
+                catalog_name.ptr(),
+                catalog_name.len_char().try_into().unwrap(),
+                schema_name.ptr(),
+                schema_name.len_char().try_into().unwrap(),
+                table_name.ptr(),
+                table_name.len_char().try_into().unwrap(),
+                table_type.ptr(),
+                table_type.len_char().try_into().unwrap(),
             )
-            .into_sql_result("SQLTablesW")
+            .into_sql_result("SQLTables")
         }
     }
 
